package F5::ASMConfig::Entity::Charset;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::Charset/;

#--------------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;

    my $dbh = $self->{dbh};
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    $store_obj->{charset} = $existing_obj->{charset};
    for my $metachar (@{$store_obj->{metachars}}) {
        if ($metachar->{metachar} == 0) {
            $metachar->{flg_is_allowed} = 0;
        }
        substr($store_obj->{charset}, $metachar->{metachar}, 1) = $metachar->{flg_is_allowed};
    }

    if ($self->get_policy_encoding_type(%params) == F5::ASMConfig::EncodingType::ENCODING_TYPE_MULTI_BYTE() &&
        $self->get_new_value(%params, field_name => 'charset_type') != F5::ASMConfig::CharsetType::CHARSET_TYPE_HEADER()) {

        # Always set all high ASCII metachars to be allowed for non-header multi-byte encodings -- in particular, utf-8.
        # This intentionally bypasses high ASCII validation (as encountered in BZ520796 and BZ494493).
        $store_obj->{charset} = substr($store_obj->{charset}, 0, 128) . '1' x 128;
    }
}

#--------------------------------------------------------------------------

sub map_fields_to_entity {
    my($class, %params) = @_;

    my $rec = $params{rec};
    my $encoding_type = $class->get_policy_encoding_type(%params);
    my $location = 0;

    if($rec->{charset}) {
        for my $digit ( split ( //, $rec->{charset})) {
            # with the exception in the case of HEADER type charset:
            #     if this is a MULTI_BYTE encoding - return only the first 128 (0-127) metachars
            if (defined $encoding_type && $location > 127 &&
                $rec->{charset_type} != F5::ASMConfig::CharsetType::CHARSET_TYPE_HEADER() &&
                $encoding_type == F5::ASMConfig::EncodingType::ENCODING_TYPE_MULTI_BYTE()
               ) {
                last;
            }
            push (@{$rec->{metachars}}, F5::ASMConfig::MetacharOverride->new({metachar => $location, flg_is_allowed => $digit}));
            $location++;
        }
    }

    $class->SUPER::map_fields_to_entity(%params);
}

#----------------------------------------------------------------------

sub get_policy_encoding_type {
    my($self, %params) = @_;

    my $rec = $params{rec};
    my $policy = $params{policy};
    my $logger = $params{logger};
    my $dbh = $params{dbh};

    my $encoding_type = F5::ASMConfig::EncodingType::ENCODING_TYPE_MULTI_BYTE();
    if ($rec->{encoding_type}) {
        $encoding_type = $rec->{encoding_type};
    } elsif ($policy) {
        $policy = F5::ASMConfig::Cache::get_cached_policy(%params, policy => $policy, dbh => $dbh); #Ensure policy is full
        $encoding_type = F5::ASMConfig::Cache::get_cached_encoding_type(%params,
                                                                        encoding_name => $policy->{encoding},
                                                                        dbh => $dbh,
                                                                        logger => $logger,
                                                                       );
    }
    return $encoding_type;
}

#----------------------------------------------------------------------

my %charset_type_to_section_type = ( F5::ASMConfig::CharsetType::CHARSET_TYPE_HEADER() => F5::ASMConfig::SectionType::SECTION_TYPE_HEADERS(),
                                     F5::ASMConfig::CharsetType::CHARSET_TYPE_URI() => F5::ASMConfig::SectionType::SECTION_TYPE_URLS(),
                                     F5::ASMConfig::CharsetType::CHARSET_TYPE_PARAM_NAME() => F5::ASMConfig::SectionType::SECTION_TYPE_PARAMETERS(),
                                     F5::ASMConfig::CharsetType::CHARSET_TYPE_PARAM_VALUE() => F5::ASMConfig::SectionType::SECTION_TYPE_PARAMETERS(),
                                     F5::ASMConfig::CharsetType::CHARSET_TYPE_XML_PROFILE() => F5::ASMConfig::SectionType::SECTION_TYPE_CONTENT_PROFILES(),
                                     F5::ASMConfig::CharsetType::CHARSET_TYPE_JSON_PROFILE() => F5::ASMConfig::SectionType::SECTION_TYPE_CONTENT_PROFILES(),
                                     F5::ASMConfig::CharsetType::CHARSET_TYPE_GWT_PROFILE() => F5::ASMConfig::SectionType::SECTION_TYPE_CONTENT_PROFILES(),
                                     F5::ASMConfig::CharsetType::CHARSET_TYPE_PLAIN_TEXT_PROFILE() => F5::ASMConfig::SectionType::SECTION_TYPE_CONTENT_PROFILES(),
                                    );

sub get_section_type {
    my($self, %params) = @_;
    my $section_type = $self->SUPER::get_section_type(%params);
    if (!$section_type && $charset_type_to_section_type{$self->{charset_type}}) {
        $section_type = $charset_type_to_section_type{$self->{charset_type}};
    }
    return $section_type;
}

1;
