package F5::ASMConfig::Entity::Cookie;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::Cookie/;

#----------------------------------------------------------------------

sub update_many_as_iterative {
    return 1;
}

#----------------------------------------------------------------------

sub delete_many_as_iterative {
    return 1;
}

#----------------------------------------------------------------------
sub before_store {
    my($self, %params) = @_;
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    $self->adjust_wildcards(%params);

    if (defined  $existing_obj &&
        $existing_obj->{cookie_name} &&
        $existing_obj->{cookie_name} eq "*"  &&
        $existing_obj->{is_wildcard}) {
        if ($self->has_value_changed(%params, check_fields => 'cookie_name')) {
            die "Wildcard cookie '*' cannot be renamed.\n";
        }
        elsif ($self->has_value_changed(%params, check_fields => 'is_wildcard')) {
            die "Cookie '*' can be only wildcard type.\n";
        }
    }

    if ($params{validate_value}) {
        my $same_site_attribute = $self->get_new_value(store_obj => $store_obj,
                                                       existing_obj => $existing_obj,
                                                       field_name => 'cookie_same_site_attribute', );

        my $is_secure = $self->get_new_value(store_obj => $store_obj,
                                             existing_obj => $existing_obj,
                                             field_name => 'is_secure', );

        if ($same_site_attribute == F5::ASMConfig::CookieSameSiteAttribute::COOKIE_SAME_SITE_ATT_NONE_VALUE() && !$is_secure) {
              my $secure_flag_display_name = $self->get_field_display_name_string(field_name => 'is_secure');
              my $same_site_display_name = $self->get_field_display_name_string(field_name => 'cookie_same_site_attribute');
              die "$secure_flag_display_name must be enabled when $same_site_display_name is set to 'None Value'";
        }
    }

    if ($self->is_pure_wildcard(%params)){
        $self->sync_from_pb_config(%params, 
                                   pb_config_field => "flg_cookies", 
                                   table_name => F5::ASMConfig::Entity::PBConfigCookie->db_table());
    }
}

#----------------------------------------------------------------------

sub delete {
    my ($self, %params) = @_;
    my $call_flags = $params{call_flags} || {};
    if ($self->{cookie_name} eq '*' && !$call_flags->{force_delete}) {
        die "Cannot delete '*' cookie\n";
    }
    return $self->SUPER::delete(%params);
}

#----------------------------------------------------------------------

sub is_pure_wildcard {
    my ($self, %params) = @_;
    return ($self->get_new_value(%params, field_name => 'cookie_name') &&
            $self->get_new_value(%params, field_name => 'cookie_name') eq "*" &&
            $self->get_new_value(%params, field_name => 'is_wildcard'));
}

#----------------------------------------------------------------------


1;
