package F5::ASMConfig::Entity::Header;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::Header/;

#----------------------------------------------------------------------

sub update_many_as_iterative {
    return 1;
}

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;

    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};
    $self->adjust_wildcards(%params);

    my @fields = qw(
                       header_name 
                       is_wildcard 
                       flg_check_signatures
                       flg_is_base64
                       percent_normalization
                       uri_normalization
                  );

    my %field_values;
    my %field_display_name;

    for my $field (@fields) {
        $field_values{$field} = $self->get_new_value( store_obj => $store_obj,
                                                      existing_obj => $existing_obj,
                                                      field_name => $field, );

        $field_display_name{$field} = $self->get_field_display_name_string(field_name => $field);
    }

    $store_obj->{is_wildcard} = $existing_obj->{is_wildcard} || 0 ; # User may create only explicit headers and may not change the type

    my $illegal_combined_fields_message;
    if ($field_values{flg_is_base64} && $field_values{percent_normalization}) {
        $illegal_combined_fields_message = "$field_display_name{flg_is_base64} and $field_display_name{percent_normalization} cannot be enabled simultaneously";
    }

    if ($field_values{flg_is_base64} && $field_values{uri_normalization}) {
        $illegal_combined_fields_message = "$field_display_name{flg_is_base64} and $field_display_name{uri_normalization} cannot be enabled simultaneously";
    }

    if ($params{validate_value} && $illegal_combined_fields_message) {
        die { error_message => $illegal_combined_fields_message,
              display_message => $illegal_combined_fields_message,
              error_level     => 'info',
              error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
          };
    }

    if ( $field_values{is_wildcard} ) {
        $store_obj->{is_mandatory} = 0; 
        if ( $field_values{header_name} eq '*') {
            $store_obj->{flg_is_base64} = 0;
        }
    }
    else {
        if ( $field_values{header_name} eq 'cookie') {
            $store_obj->{flg_check_signatures} = 0;
            $store_obj->{flg_is_base64} = 0;
            $store_obj->{enable_normalization_violation} = 0;
            $store_obj->{uri_normalization} = 0;
            $store_obj->{html_normalization} = 0;
            $store_obj->{enable_normalization_violation} = 0;
            $store_obj->{flg_mask_value} = 0;
        }

        if ( $field_values{header_name} eq 'authorization') {
            $store_obj->{flg_is_base64} = 0;
        }
    }

    if ( !$field_values{flg_check_signatures} ) {
        $store_obj->{enable_normalization_violation} = 0;
        $store_obj->{percent_normalization} = 0;
        $store_obj->{uri_normalization} = 0;
        $store_obj->{html_normalization} = 0;
    }
}

#----------------------------------------------------------------------------


1;

