package F5::ASMConfig::Handler::CredentialStuffingFile;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;

use F5::LockFactory;
use F5::PrepareConf::Base;
use F5::SetActive;
use F5::CredentialStuffingFile;

our @handled_functions = (
                           {
                             function_name => 'update_credential_stuffing_file',
			     filename_arguments => ['3.filename'],
			     relayable => 1,
                             write_access => 1,
                           },
                         );

#--------------------------------------------------------------------

sub update_credential_stuffing_file {
  my($self, $session, $call_flags, $place_holder, $credential_stuffing_file) = @_;

  my $dbh = $self->{dbh};
  my $logger = $self->{logger};

  my $rv = F5::ASMConfig::CredentialStuffingUpdateRes->new();

  eval {
      my $lock_file =  $F5::DbUtils::LOCK_FILE;
      my $locker = F5::LockFactory::get_locker(lock_filename => $lock_file,
					       logger => $logger);
      $locker->lock();

      F5::CredentialStuffingFile::load_credential_stuffing_file(filename => $credential_stuffing_file->{filename},
								logger => $self->{logger},
                                                                dbh => $dbh);

      my $prepareconf = F5::PrepareConf::Base->new(dbh => $dbh,
                                                   logger => $logger);

      $prepareconf->prepare_credential_stuffing_file;

      $dbh->commit unless $dbh->{AutoCommit};

      F5::SetActive::set_active(dbh => $dbh,
				logger => $logger,
				do_update_core => 1,
				send_sync => 1,
				prepare_account_tables => 0,
				asm_config_handler => $self);

      $locker->unlock();

      $rv->{completed_successfully} = 1;
  };
  if ($@) {
      $rv->{completed_successfully} = 0;
      $rv->{error_msg} = "Failed to update credential stuffing file";
  }

  return { return_value => $rv };
}

#--------------------------------------------------------------------

1;
