package F5::ASMConfig::Handler::Header;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::Header;

our @handled_functions = (
                           {
                             function_name => 'delete_many_headers',
                             relayable => 1,
                           },
                           {
                             function_name => 'delete_header',
                             relayable => 1,
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::Header';
#----------------------------------------------------------------------

sub delete_many_headers {
    my($self, $session, $call_flags, $policy, $header_criteria) = @_;

    my $local_call_flags = F5::ASMConfig::CallFlags->new($call_flags);

    if (!defined $local_call_flags->{search_filters}){
       $local_call_flags->{search_filters} = [];
    }

    push(@{$local_call_flags->{search_filters}},
                 { field_name => 'flg_is_default',
                   field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                   field_value => 0 });

    my $rv = $self->base_delete_many(entity_class => $entity_class,
                                     rpc_params => [ $session, $local_call_flags,$policy, $header_criteria ],);

    return $rv;
}

#----------------------------------------------------------------------
sub delete_header {
  my($self, $session, $call_flags, $policy,$header) = @_;

  my $deleted_header = F5::ASMConfig::Entity::Header->new(  dbh => $self->{dbh},
                                                            policy => $policy,
                                                            get_criteria => $header );


  if ( $deleted_header->{flg_is_default} ){
     return $self->handle_error( error_code =>  F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
                                 error_thrown => "Default header '$header->{header_name}' cannot be deleted.\n",
                                 error_level  => "info",
                                 thrift_obj => $header,
                                 call_flags => $call_flags,
                               );
  }

  return $self->base_delete(rpc_params => [$session, $call_flags, $policy,$header],
                            entity_class => $entity_class,
                           );
}
#----------------------------------------------------------------------
1;
