package F5::DbFixup::CredentialStuffing;

use strict;
use warnings;
use base 'F5::DbFixup::Default';

# Package globals, handled by AUTOLOAD
our $fixup_db = 'PLC';
our $excluded_tables = ['PLC.CREDENTIAL_STUFFING_UPDATE_FILE'];

#Retain Data Protection Settings already in the system if loading a UCS from a version without those tables.
sub after_fixup {
    my ($self, $dbh, $old_dbname, $new_dbname, $logger, $params) = @_;

    my $databases = $params->{databases};

    $logger->debug("Begin Credential Stuffing fixup");

    for my $full_table (@$excluded_tables) {
	# PLC_staging/DCC_staging tables should be empty, but clear them out to be sure
        my($db_type, $table_name) = split(/\./, $full_table);
        $dbh->do(<<EndQuery, undef);
DELETE FROM $databases->{$db_type}->{staging_dbname}.$table_name
EndQuery
	$self->copy_table(dbh => $dbh,
		   old_dbname => $databases->{$db_type}->{bigip_dbname},
		   new_dbname => $databases->{$db_type}->{staging_dbname},
		   table_name => $table_name,
		   id_field_name => '',
		   logger => $logger);
    }

    $logger->debug("End Credential Stuffing fixup");
}

1;
