package F5::NATSClient;

use strict;
use warnings;

use Net::NATS::Client;
use File::Slurp;
use MIME::Base64;
use JSON;
use Data::Dumper;

#----------------------------------------------------------------------

sub new {
    my($class, %params) = @_;
    my $self = bless {}, $class;

    $self->init(%params);
    return $self;
}

#----------------------------------------------------------------------

sub init {
    my($self, %params) = @_;
    for my $k (%params) {
        $self->{$k} = $params{$k};
    }

    my $username_prefix = '';
    if ($self->{username}) {
        $username_prefix = $self->{username};
        if ($self->{password}) {
            $username_prefix .= ":$self->{password}";
        }
        $username_prefix .= '@';
    }

    # CPAN module Net::NATS::Client uses module URI under the hood in order to parse the URI. The
    # username and password cannot be extracted if an unknown protocol is used. Thus, although the
    # actual protocol is NATS internal protocol, this is listed as "ftp" in order for URI parsing to
    # work correctly.
    $self->{client} = Net::NATS::Client->new(uri => 'ftp://' . "$username_prefix$self->{host}:$self->{port}");
    $self->{client}->connect() or die $!;

    my $subscription = $self->{client}->subscribe($self->{subject}, sub {
                                                      my ($message) = @_;
                                                      $self->handle_message($message);
                                                  });
}

#----------------------------------------------------------------------

sub run {
    my($self) = @_;
    while (1) {
        $self->{client}->wait_for_op();
    }
}

#----------------------------------------------------------------------

sub handle_message {
    my($self, $message) = @_;
    my $proto_struct = $self->{proto_base_struct};
    my $message_rec = $proto_struct->decode($message->data);

    my $json = JSON->new->allow_nonref->convert_blessed;
    print $json->pretty->encode($message_rec);
}

#----------------------------------------------------------------------

sub UNIVERSAL::TO_JSON { return { %{ shift() } }; }

1;
