package F5::ServerLookup;

use strict;
use warnings;

use Socket qw(:DEFAULT inet_pton inet_ntop);
use F5::SystemUtils;

sub get_ip_address {
  my($hostname,$timeout) = @_;

  # Return if IP address is already provided
  return $hostname if ($hostname =~ /^[\d.]+$/);

  my $ret_hostname;
  my $res = undef;
  eval {
      local $SIG{ALRM} = sub { die "Timeout triggered\n" };
      if ($timeout) {
          alarm $timeout;
      }

      $ret_hostname = gethostbyname($hostname);

      if ($timeout) {
          alarm 0;
      }

      if (defined $ret_hostname) {
          $ret_hostname = inet_ntop(AF_INET, $ret_hostname);
      }
      elsif (-e '/usr/local/bin/find-activate.pl') {
          my @params;
          push ( @params, '/usr/local/bin/find-activate.pl','-l',$hostname);
          $res =  F5::SystemUtils::run_system_tool( command_line_params => \@params,
                                                    timeout => $timeout,
                                                  );
          if ($res && $res->{completed_successfully}) {
              $ret_hostname = $res->{standard_output};
              chomp($ret_hostname);
          }
      }
      else {
          $ret_hostname = $hostname;
      }

  };

  my $ex = $@;
  if ($timeout) {
      alarm 0;
  }
  if ($ex) {
      die "Can't get hostname IP address\n";
  }

  if ($res && ! $res->{completed_successfully}) {
      die "Can't get hostname IP address ($res->{standard_error})\n";
  }

  return $ret_hostname;

}

1;
